@@ -1,7 +1,6 @@
 #!/usr/bin/env python3
 # pylint: disable=missing-docstring,not-an-iterable,too-many-locals,too-many-arguments,invalid-name
 
-import os
 import collections
 import itertools
 import copy
@@ -11,7 +10,7 @@ import singer.schema
 from singer import utils, metadata, get_bookmark
 from singer.schema import Schema
 from singer.catalog import Catalog, CatalogEntry
-from c8connector import C8Connector, ConfigProperty, Sample
+from c8connector import C8Connector, ConfigProperty, Sample, SchemaAttribute, SchemaAttributeType
 import c8_source_oracle.db as orc_db
 import c8_source_oracle.sync_strategies.log_miner as log_miner
 import c8_source_oracle.sync_strategies.full_table as full_table
@@ -587,26 +586,40 @@ class OracleSourceConnector(C8Connector):
       """Validate given configurations against the connector.
       If invalid, throw an exception with the cause.
       """
-      
-      config = {
-         'host': integration['host'],
-         'user': integration['user'],
-         'password': integration['password'],
-         'port': integration['port'],
-         'service_name': integration['service_name'],
-         'pdb_name': integration['pdb_name'],
-         'multitenant': integration['multitenant'],
-      }
-      
-      filter_schemas_prop = integration['filter_schemas']
-      filter_schemas = []
-      if filter_schemas_prop:
-         filter_schemas = filter_schemas_prop.split(',')
+      config, filter_schemas = self.get_config(integration)
       do_discovery(config, filter_schemas)
 
+   def schemas(self, integration: dict) -> list[Schema]:
+      """Get supported schemas using the given configurations."""
+      config, filter_schemas = self.get_config(integration)
+      catalog = do_discovery(config, filter_schemas)
+      results = []
+      for stream in catalog.streams:
+         s_attribs = []
+         s_schema = stream.schema
+         for k, v in s_schema.properties.items():
+            t = v.type[-1]
+            s_attribs.append(SchemaAttribute(k, self.get_attribute_type(t)))
+         results.append(Schema(stream.stream, s_attribs))
+      return results
+
    def samples(self, integration: dict) -> list[Sample]:
       """Fetch sample data using the given configurations."""
-      return []
+      config, filter_schemas = self.get_config(integration)
+      catalog = do_discovery(config, filter_schemas)
+      results = []
+      for stream in catalog.streams:
+         s_attribs = []
+         s_schema = stream.schema
+         for k, v in s_schema.properties.items():
+            t = v.type[-1]
+            s_attribs.append(SchemaAttribute(k, self.get_attribute_type(t)))
+         schema = Schema(stream.stream, s_attribs)
+         results.append(Sample(
+               schema=schema,
+               data=orc_db.fetch_samples(config, stream)
+            )
+         )
 
    def config(self) -> list[ConfigProperty]:
       """Get configuration parameters for the connector."""
@@ -629,6 +642,40 @@ class OracleSourceConnector(C8Connector):
       """
       return ['catalog', 'discover', 'state']
 
+   @staticmethod
+   def get_attribute_type(source_type: str) -> SchemaAttributeType:
+      if source_type == 'string':
+         return SchemaAttributeType.STRING
+      elif source_type == 'integer':
+         return SchemaAttributeType.INT
+      elif source_type == 'boolean':
+         return SchemaAttributeType.BOOLEAN
+      elif source_type == 'number':
+         return SchemaAttributeType.DOUBLE
+      else:
+         return SchemaAttributeType.OBJECT
+
+   @staticmethod
+   def get_config(integration: dict):
+      try:
+         config = {
+            'host': integration['host'],
+            'user': integration['user'],
+            'password': integration['password'],
+            'port': integration['port'],
+            'service_name': integration['service_name'],
+            'pdb_name': integration['pdb_name'],
+            'multitenant': integration['multitenant'],
+         }
+         
+         filter_schemas_prop = integration['filter_schemas']
+         filter_schemas = []
+         if filter_schemas_prop:
+            filter_schemas = filter_schemas_prop.split(',')
+         return config, filter_schemas
+      except KeyError as e:
+         raise KeyError(f'Integration property `{e}` not found.')
+
 def main():
     try:
         main_impl()
